@@ -5,7 +5,7 @@
 
 OLDPWD=$(pwd)
 
-VERSION="0.4.0"
+VERSION="0.5.1"
 NAME="kig"
 
 TEMPDIR="/tmp/$NAME-package-temp"
@@ -14,11 +14,11 @@ mkdir $TEMPDIR
 cd $TEMPDIR
 
 cd $TEMPDIR
-~domi/src/cvs2dist \
+~domi/src/kdesdk/scripts/cvs2dist \
 	--name "$NAME" \
 	--version "$VERSION" \
 	--log="$TEMPDIR/log" \
-	~/src/test/kdeedu kig
+	~/src/package/kdeedu kig
 
 cd $OLDPWD
 
